/* CVS: CobaltVault SERVER:\\halo TREE:\abode_mainline
 *       _    ____   ___  ____  _____ 
 *      / \  | __ ) / _ \|  _ \| ____|      Advanced
 *     / _ \ |  _ \| | | | | | |  _|        Behavior
 *    / ___ \| |_) | |_| | |_| | |___       Oriented
 *   /_/   \_\____/ \___/|____/|_____|      Design
 *         www.cobaltsoftware.net           Environment
 *
 * PRODUCED FOR:      University of Bath / Boeing
 * PAYMENT:           On Delivery   
 * LICENSING MODEL:   Unrestricted distribution (Post Delivery)
 * COPYRIGHT:         Client retains copyright.
 *
 * This program and all the software components herein are
 * released as-is, without warranties regarding function,
 * correctness or any other aspect of the components.
 * Steven Gray, Cobalt Software, it's subcontractors and
 * successors may not be held liable for any damage caused 
 * to computers, business or other property through use of 
 * or misuse of this software.
 *
 * Upon redistribution of the program, all notices of
 * copyrights, both of the software provider and the 
 * client must be retained.
 */                               
package abode.lib;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * SplashScreen is the class for pre-initialization of the various classes used by
 * the software at launch time. Using Class.forName(string) it ensures each of the
 * classes are pre-initialized when the application starts, so that various delays
 * waiting for resources and initialization do not occur during general program use,
 * where they may beunexpected by the user.
 * 
 * @author  CobaltSoftware (abode.devteam@cobaltsoftware.net)
 * @version 1.0
 */
public class SplashScreen extends javax.swing.JFrame {
    // List of all classes within the application that are to be pre-initialized
    private String[] classes = { "abode.lib.ActionElement", "abode.lib.ActionPattern", "abode.lib.Competence",
                             "abode.lib.CompetenceElement", "abode.lib.Configuration", "abode.lib.ConsoleWriter",
                             "abode.lib.CurrentLineHighlighter", "abode.lib.Documentation", "abode.lib.DotLapReader",
                             "abode.lib.DotLapWriter", "abode.lib.DriveCollection", "abode.lib.DriveElement", 
                             "abode.lib.IEditableElement", "abode.lib.ILAPReader", "abode.lib.IListOrganiser",
                             "abode.lib.INamedElement", "abode.lib.IPrimitiveReader", "abode.lib.IRenderer",
                             "abode.lib.JAbode", "abode.lib.JColorSelectButton", 
                             "abode.lib.JColorSelectForm", "abode.lib.JDiagram", "abode.lib.JEditorWindow", "abode.lib.JEngineInput",
                             "abode.lib.JOptionsScreen", "abode.lib.JTreeNode", "abode.lib.LearnableActionPattern","abode.lib.LispBlob",
                             "abode.lib.LispPrimitiveParser", "abode.lib.PrimitiveManager", "abode.lib.PrintDiagramRenderer",
                             "abode.lib.StandardDiagramRenderer", "abode.lib.TimeUnit", "abode.lib.VerticalListOrganiser"};
              
    // Our lovely splash image
    private ImageIcon icon = new ImageIcon(getClass().getResource("/abode/images/splash/monkey.png"));
    
    /**
     * Constructs the splash screen and starts the various actions rolling. 
     * Perhaps in poor form for JFrame constructors, this particular class 
     * makes itself visible, but it's usefulness in the application is so
     * restricted that this was considered an acceptable compromise.
     **/
    public SplashScreen() {
        // NetBeans Form Designer
        initComponents();
               
        // Centre us on the screen
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension labelSize = getPreferredSize();
        setLocation(screenSize.width/2 - (labelSize.width/2), screenSize.height/2 - (labelSize.height/2));
        
        // Make us visible and bring us forward
        setVisible(true);
        toFront();
       
        // Show all classes
        doInitialization();
    }
    
    /**
     * Perform the one-time initialization of all classes by using the Java class loader
     * and a list of named classes from our system.
     **/
    private void doInitialization() {
        // Set the correct bounds on the progress bar
        jProgressBar1.setMinimum(0);
        jProgressBar1.setMaximum(classes.length);
        
        // For each of the classes we're going to initialize
        for (int x = 0; x < classes.length; x++) {
            // Move progress along
            jProgressBar1.setValue(x);

            // Show the class we're about to load and wait 1/20th of a second
            jProgressBar1.setBackground(Color.green);
            jProgressBar1.setString("Initializing " + classes[x]);
            repaint();
            try { Thread.sleep(20); } catch (Exception e) { }
            
            // Initialize the class
            try {
                Class.forName(classes[x]);
                
                // Success, notify user
                jProgressBar1.setString("Initialized " + classes[x]);
                repaint();
                
                // Wait briefly, do the next one
                //try { Thread.sleep(40); } catch (Exception e) { }
            } catch (Exception e) {
                jProgressBar1.setBackground(Color.RED);
                jProgressBar1.setString("Failure initializing " + classes[x]);
                repaint();
                try { Thread.sleep(3000); } catch (Exception f) { }
            }
        }
       
        // Get rid of this frame
        dispose();
        setVisible(false);
        
        // Show the main application (Which should spring up quite rapidly now)
        new JAbode().setVisible(true);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        splashPanel = new javax.swing.JPanel();
        jProgressBar1 = new javax.swing.JProgressBar();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("\n");
        setForeground(java.awt.Color.white);
        setUndecorated(true);
        splashPanel.setLayout(null);

        splashPanel.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        splashPanel.setMaximumSize(new java.awt.Dimension(349, 200));
        splashPanel.setMinimumSize(new java.awt.Dimension(349, 200));
        splashPanel.setPreferredSize(new java.awt.Dimension(349, 200));
        jProgressBar1.setString("Initialising Classes");
        jProgressBar1.setStringPainted(true);
        splashPanel.add(jProgressBar1);
        jProgressBar1.setBounds(46, 155, 269, 14);

        getContentPane().add(splashPanel, java.awt.BorderLayout.CENTER);

        pack();
    }
    // </editor-fold>//GEN-END:initComponents
       
    /**
     * Evil overloaded paint method that actually skips the rendering of the panel itself, but draws
     * directly onto the form background then renders the children of the panel. This particular 
     * workaround prevents the need for us to create a new class just to deal with a simple bit of 
     * background rendering.
     *
     * @param g Graphics object for rendering with.
     **/
    public void paint(Graphics g) {
        // Draw background image
        g.drawImage(icon.getImage(), 0, 0, getWidth(), getHeight(), null, null);

        // EVIL: Call containing panels child painting method.
        splashPanel.paintComponents(g);
    }
           
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JProgressBar jProgressBar1;
    private javax.swing.JPanel splashPanel;
    // End of variables declaration//GEN-END:variables
    
}
